#include <WString.h>
#include <AF_Wave.h>
#include <avr/pgmspace.h>
#include "util.h"
#include "wave.h"

#define maxLength 16
#define ID  0
#define CMD 2
#define AR1 4
#define AR2 8
#define FILTER_SHIFT 3

AF_Wave card;
File f;
Wavefile wave;

// Modify the following according to your needs
long ligthDelay = 400;
boolean debug = true;
int channels[]={6,7,8,15,16,17,18,19}; // determine the digital outputs that will drive the SSRs
int channelsLenght = 8; 
///////////////////////////////////////////////

int32_t filter_reg;
int16_t filter_input;
int16_t filter_output;
uint16_t samplerate;
int volPin = 0;
int val; 
long randChannel;
long randLightmode;
long volume;
long v2;
int queue;
int mode = 0;
int queueMode = 0;
char *track;
int count = 0;
unsigned long time;
unsigned long randDuration = 0;
int inCount;
boolean answer = true;
boolean commandComplete = false;

String inString = String(maxLength);
String tmpTrack = String(maxLength);  
String command = String(maxLength);



void setup() {
  Serial.begin(9600);
  //Serial.println("Wave test!");
  // pins used by the Wave Shield
  for (int i = 2; i <= 5; ++i)
    pinMode(i, OUTPUT);

  pinMode(9, OUTPUT);  // pin used to turn on fm transmitter

  for (int i = 0; i < channelsLenght; i++) 
    pinMode(channels[i], OUTPUT);     

  if (!card.init_card()) {
    if (debug)putstring_nl("Card init. failed!"); 
    return;
  }
  if (!card.open_partition()) {
    if (debug)putstring_nl("No partition!"); 
    return;
  }
  if (!card.open_filesys()) {
    if (debug)putstring_nl("Couldn't open filesys"); 
    return;
  }
  if (!card.open_rootdir()) {
    if (debug)putstring_nl("Couldn't open dir"); 
    return;
  }
  
  if (debug)putstring_nl("Files found:");
  ls();

//  if (debug)putstring_nl("turn on FM transmiter");
//  delay(500);
//  digitalWrite(9, HIGH);
//  delay(200);
//  digitalWrite(9, LOW);

  command = "";
}

void ls() {
  char name[13];
  card.reset_dir();
  if (debug)putstring_nl("Files found:");
  while (1) {
    if (!card.get_next_name_in_dir(name)) {
      card.reset_dir();
      return;
    }
    // print files
    if (debug)Serial.println(name);
  }
}

uint8_t tracknum = 0;

void loop() { 
  uint8_t i, r;
  char c, name[15];

  card.reset_dir();
  // scroll through the files in the directory
  for (i=0; i<tracknum+1; i++) {
    r = card.get_next_name_in_dir(name);
    if (!r) {
      // ran out of tracks! start over
      tracknum = 0;
      return;
    }
  }
  //putstring("\n\rPlaying ");  
  //putstring("\n"); Serial.print(name);
  // reset the directory so we can find the file
  card.reset_dir();   
  playComplete(name);
  tracknum++;

}

void playComplete(char *name) {
  uint16_t potval;
  uint32_t newsamplerate;
  char c;

  // check if we should play from the remote queue
//  if (queue == 1){
//    // play track from remote queue
//    playFile(track);    
//    track = "";
//    queue = 0;
//    mode = queueMode;
//  }
//  else{
    playFile(name);
  }
  
  // set samplerate
  samplerate = wave.dwSamplesPerSec;    
  int isPlayingCounter = 1;


  // main loop
  while (wave.isplaying) {          

    isPlayingCounter++;
    // listen for new commands
    if(Serial.available() > 0) {   
      char inChar = Serial.read();
      // check for end of command char ';'
      if(inChar == 59 || inChar == 10 || inChar == 13){
        inString = command;
        command = "";
        mode = 0;
        commandComplete = true;
      } 
      else {
        command.append(inChar);
      }      
    }

    if (commandComplete == true || mode > 0) {  
      //Serial.println(inString);
      if(inString.charAt(0) == 'n' && mode == 0){ // pin mode
        if (wave.isplaying) {// already playing something, so stop it!
          if (debug)putstring("Next track");
          commandComplete = false;
          inString = "";
          wave.stop(); // stop it                
        }  
      }    

      if(inString.charAt(0) == 'd'){ // pin mode
          char *tmpligthDelay;
          tmpligthDelay[0] = inString.charAt(1);
          tmpligthDelay[1] = inString.charAt(2);
          tmpligthDelay[2] = inString.charAt(3);
            if (isNumeric(inString.charAt(4))){
              tmpligthDelay[3] = inString.charAt(4);
            }
          ligthDelay = atoi(tmpligthDelay);
      }  

      if(inString.charAt(0) == 'q' && isNumeric(inString.charAt(1)) && mode == 0){
        tmpTrack = "";
        track = "";
        tmpTrack.append(inString.charAt(1));
        if (isNumeric(inString.charAt(2)))
            tmpTrack.append(inString.charAt(2));
        tmpTrack.append(".WAV");
        track = tmpTrack;
        if (inString.charAt(3) == 'm'){
          char modeChar = inString.charAt(4);
          queueMode = atoi(&modeChar);
          //putstring("Queue ligth mode: ");
          //Serial.println(queuemode, DEC);
        }
        //putstring("On queue: ");
        //Serial.println(track);
        commandComplete = false;
        inString = "";
        queue = 1;
        //wave.stop();
      }

      if((inString.charAt(3) == 'm' && inString.charAt(4) == '1') || mode == 1){ 
        doVuMeter();
        mode = 1;
      }
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '2') || mode == 2){
        doRandomChannel();
        mode = 2;
      }
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '3') || mode == 3){
        doSequence();
        mode = 3;
      }
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '4') || mode == 4){ 
        doMerge();
        mode = 4;
      }
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '5') || mode == 5){
        doAscend();
        mode = 5;
      }    
      else if((inString.charAt(3) == 'm' && inString.charAt(6) == '6') || mode == 6){
        doDescend();
        mode = 6;
      }
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '7') || mode == 7){
        doSplit();
        mode = 7;
      }
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '8' && isNumeric(inString.charAt(5))) || mode == 8 ){     
        doAllChannels(inString.charAt(5));
        mode = 8;
      }    
      else if((inString.charAt(3) == 'm' && inString.charAt(4) == '9' && isNumeric(inString.charAt(5)) && isNumeric(inString.charAt(6))) || mode == 9){        
        doChannel(inString.charAt(5),inString.charAt(6));
        mode = 9;
      }else{
      randomMode(isPlayingCounter);
    }
      
    }else{
      randomMode(isPlayingCounter);
    }

    delay(100); //100 looks good

  }// end while wave.isplaying

  card.close_file(f);

}

void randomMode(int i){
    if (debug)Serial.println("Random Channel");
    time = millis();
    
    if (time > randDuration){
      randDuration = time + (10 * 1000) ;
      randLightmode = random(1,1);
    }
    
    switch (randLightmode){
    case 1:
      doVuMeter();
      break;
//    case 2:
//      doRandomChannel();
//      break;
//    case 3:
//     doSequence();
//      break;
//    case 4:    
//      doAscend();
//      break;
//    case 5:
//      doDescend();
//      break;
//    case 6:
//      doSplit();
//      break;
//    case 7:
//      doMerge();
//      break;       
    }
  
}


void playFile(char *name) {
  f = card.open_file(name);
  if (!f) {
    if (debug)putstring_nl(" Couldn't open file"); 
    return;
  }
  if (!wave.create(f)) {
    if (debug)putstring_nl(" Not a valid WAV"); 
    return;
  }
  // ok time to play!
  wave.play();


  Serial.print(name[0]);
  if (isNumeric(name[1])){
    Serial.print(name[1]);
  }

  
}

void doAllChannels(char state){
  if (debug)Serial.println("ALL Channels");
  int intState = atoi(&state); 

  for (int i = 0; i < channelsLenght; i++) {
    digitalWrite(channels[i], intState);
  }  
}

void doChannel(char channel, char state){ 
  if (debug)Serial.println("Single Channel");
  int intChannel = atoi(&channel); 
  int intState = atoi(&state); 
  digitalWrite(channels[intChannel], intState);
}

void doRandomChannel(){ 
  if (debug)Serial.println("Random Channel");
  for (int i = 0; i < 3; i++) {
    randChannel = random(0,8);
    digitalWrite(channels[randChannel], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[randChannel], LOW);
  }  
}

void doVuMeter(){
  if (debug)Serial.println("Vu Meter");
  volume = 0;
  for (int i=0; i<8; i++) {
    v2 = analogRead(0) - 350;
    if (v2 < 0) 
      v2 *= -1;
    if (v2 > volume)
      volume = v2;
    delay(5);
  }

  volume = volume / 30;
  //putstring("\n");Serial.print(volume);

  for (int i = 0; i < channelsLenght; i++) {
    if (volume > i +3 ){
      digitalWrite(channels[i], HIGH);
    }
    else{
      digitalWrite(channels[i], LOW);
    }  
  }

}

void doSequence(){
  if (debug)Serial.println("Sequence");
  for (int i = 0; i < channelsLenght; i++) {
    digitalWrite(channels[i], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[i], LOW);
    delay(ligthDelay);  
  }

  for (int i =  channelsLenght - 1; i >= 0; i--) {
    digitalWrite(channels[i], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[i], LOW);
    delay(ligthDelay);  
  }

}

void doMerge(){
  if (debug)Serial.println("Merge");
  int rev = 7;
  for (int i = 0; i < 4; i++) {
    digitalWrite(channels[i], HIGH);
    digitalWrite(channels[rev - i], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[i], LOW);
    digitalWrite(channels[rev - i], LOW);
    delay(ligthDelay);  
  }

}

void doSplit(){
  if (debug)Serial.println("Split");
  int rev = 3;
  int inc = 1;
  for (int i = 3; i >= 0; i--) {
    digitalWrite(channels[i], HIGH);
    digitalWrite(channels[rev + inc], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[i], LOW);
    digitalWrite(channels[rev + inc], LOW);
    delay(ligthDelay);  
    inc++;
  }

}

void doDescend(){
  if (debug)Serial.println("Descend");
  for (int i =  channelsLenght - 1; i >= 0; i--) {
    digitalWrite(channels[i], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[i], LOW);
    delay(ligthDelay);  
  }

}

void doAscend(){
  if (debug)Serial.println("Ascend");
  for (int i = 0; i < channelsLenght; i++) {
    digitalWrite(channels[i], HIGH);
    delay(ligthDelay);
    digitalWrite(channels[i], LOW);
    delay(ligthDelay);  
  }
}

boolean isNumeric(char character){
  boolean ret = false;
  //int number = atoi(&character); 
  if(character >= 48 && character <= 75){
  //if (number > 0){  
    ret = true;
  }
  return ret;
}

